#ifndef __SCRUFFY__CPP_COND_PARGEN_H__
#define __SCRUFFY__CPP_COND_PARGEN_H__

#include <mycpp/string.h>

#include <pargen/parser_element.h>
#include <pargen/grammar.h>

namespace Scruffy {

class CppCondElement : public Pargen::ParserElement
{
public:
    enum Type {
        _Identifier,
        _Literal,
        _PrimaryExpression,
        _UnaryExpression,
        _UnaryOperator,
        _MultiplicativeExpression,
        _AdditiveExpression,
        _ShiftExpression,
        _RelationalExpression,
        _EqualityExpression,
        _AndExpression,
        _ExclusiveOrExpression,
        _InclusiveOrExpression,
        _LogicalAndExpression,
        _LogicalOrExpression,
        _ConditionalExpression,
        _Grammar
    };

    const Type cpp_cond_element_type;

    CppCondElement (Type type)
        : cpp_cond_element_type (type)
    {
    }
};

class CppCond_Identifier;
class CppCond_Literal;
class CppCond_PrimaryExpression;
class CppCond_UnaryExpression;
class CppCond_UnaryOperator;
class CppCond_MultiplicativeExpression;
class CppCond_AdditiveExpression;
class CppCond_ShiftExpression;
class CppCond_RelationalExpression;
class CppCond_EqualityExpression;
class CppCond_AndExpression;
class CppCond_ExclusiveOrExpression;
class CppCond_InclusiveOrExpression;
class CppCond_LogicalAndExpression;
class CppCond_LogicalOrExpression;
class CppCond_ConditionalExpression;

class CppCond_Identifier : public CppCondElement
{
public:
    Pargen::ParserElement_Token *any_token;

    CppCond_Identifier ()
        : CppCondElement (CppCondElement::_Identifier)
          , any_token (NULL)
    {
    }
};

void dump_cpp_cond_Identifier (CppCond_Identifier const *, MyCpp::Size nest_level = 0);

class CppCond_Literal : public CppCondElement
{
public:
    Pargen::ParserElement_Token *any_token;

    CppCond_Literal ()
        : CppCondElement (CppCondElement::_Literal)
          , any_token (NULL)
    {
    }
};

void dump_cpp_cond_Literal (CppCond_Literal const *, MyCpp::Size nest_level = 0);

class CppCond_PrimaryExpression : public CppCondElement
{
public:
    enum Type {
        _Literal,
        _Braces
    };

    const Type primary_expression_type;

    CppCond_PrimaryExpression (Type type)
        : CppCondElement (CppCondElement::_PrimaryExpression),
          primary_expression_type (type)
    {
    }
};

class CppCond_PrimaryExpression_Literal : public CppCond_PrimaryExpression
{
public:
    CppCond_Literal *literal;

    CppCond_PrimaryExpression_Literal ()
        : CppCond_PrimaryExpression (CppCond_PrimaryExpression::_Literal)
          , literal (NULL)
    {
    }
};

class CppCond_PrimaryExpression_Braces : public CppCond_PrimaryExpression
{
public:
    CppCond_ConditionalExpression *expression;

    CppCond_PrimaryExpression_Braces ()
        : CppCond_PrimaryExpression (CppCond_PrimaryExpression::_Braces)
          , expression (NULL)
    {
    }
};

void dump_cpp_cond_PrimaryExpression (CppCond_PrimaryExpression const *, MyCpp::Size nest_level = 0);

class CppCond_UnaryExpression : public CppCondElement
{
public:
    enum Type {
        _Defined,
        _DefinedBraces,
        _Expression,
        _UnaryOperator
    };

    const Type unary_expression_type;

    CppCond_UnaryExpression (Type type)
        : CppCondElement (CppCondElement::_UnaryExpression),
          unary_expression_type (type)
    {
    }
};

class CppCond_UnaryExpression_Defined : public CppCond_UnaryExpression
{
public:
    CppCond_Identifier *identifier;

    CppCond_UnaryExpression_Defined ()
        : CppCond_UnaryExpression (CppCond_UnaryExpression::_Defined)
          , identifier (NULL)
    {
    }
};

class CppCond_UnaryExpression_DefinedBraces : public CppCond_UnaryExpression
{
public:
    CppCond_Identifier *identifier;

    CppCond_UnaryExpression_DefinedBraces ()
        : CppCond_UnaryExpression (CppCond_UnaryExpression::_DefinedBraces)
          , identifier (NULL)
    {
    }
};

class CppCond_UnaryExpression_Expression : public CppCond_UnaryExpression
{
public:
    CppCond_PrimaryExpression *expression;

    CppCond_UnaryExpression_Expression ()
        : CppCond_UnaryExpression (CppCond_UnaryExpression::_Expression)
          , expression (NULL)
    {
    }
};

class CppCond_UnaryExpression_UnaryOperator : public CppCond_UnaryExpression
{
public:
    CppCond_UnaryOperator *unaryOperator;
    CppCond_UnaryExpression *expression;

    CppCond_UnaryExpression_UnaryOperator ()
        : CppCond_UnaryExpression (CppCond_UnaryExpression::_UnaryOperator)
          , unaryOperator (NULL)
          , expression (NULL)
    {
    }
};

void dump_cpp_cond_UnaryExpression (CppCond_UnaryExpression const *, MyCpp::Size nest_level = 0);

class CppCond_UnaryOperator : public CppCondElement
{
public:
    enum Type {
        _Plus,
        _Minus,
        _Not,
        _Inversion
    };

    const Type unary_operator_type;

    CppCond_UnaryOperator (Type type)
        : CppCondElement (CppCondElement::_UnaryOperator),
          unary_operator_type (type)
    {
    }
};

class CppCond_UnaryOperator_Plus : public CppCond_UnaryOperator
{
public:

    CppCond_UnaryOperator_Plus ()
        : CppCond_UnaryOperator (CppCond_UnaryOperator::_Plus)
    {
    }
};

class CppCond_UnaryOperator_Minus : public CppCond_UnaryOperator
{
public:

    CppCond_UnaryOperator_Minus ()
        : CppCond_UnaryOperator (CppCond_UnaryOperator::_Minus)
    {
    }
};

class CppCond_UnaryOperator_Not : public CppCond_UnaryOperator
{
public:

    CppCond_UnaryOperator_Not ()
        : CppCond_UnaryOperator (CppCond_UnaryOperator::_Not)
    {
    }
};

class CppCond_UnaryOperator_Inversion : public CppCond_UnaryOperator
{
public:

    CppCond_UnaryOperator_Inversion ()
        : CppCond_UnaryOperator (CppCond_UnaryOperator::_Inversion)
    {
    }
};

void dump_cpp_cond_UnaryOperator (CppCond_UnaryOperator const *, MyCpp::Size nest_level = 0);

class CppCond_MultiplicativeExpression : public CppCondElement
{
public:
    enum Type {
        _Expression,
        _Multiply,
        _Divide,
        _Remainder
    };

    const Type multiplicative_expression_type;

    CppCond_MultiplicativeExpression (Type type)
        : CppCondElement (CppCondElement::_MultiplicativeExpression),
          multiplicative_expression_type (type)
    {
    }
};

class CppCond_MultiplicativeExpression_Expression : public CppCond_MultiplicativeExpression
{
public:
    CppCond_UnaryExpression *expression;

    CppCond_MultiplicativeExpression_Expression ()
        : CppCond_MultiplicativeExpression (CppCond_MultiplicativeExpression::_Expression)
          , expression (NULL)
    {
    }
};

class CppCond_MultiplicativeExpression_Multiply : public CppCond_MultiplicativeExpression
{
public:
    CppCond_MultiplicativeExpression *left;
    CppCond_UnaryExpression *right;

    CppCond_MultiplicativeExpression_Multiply ()
        : CppCond_MultiplicativeExpression (CppCond_MultiplicativeExpression::_Multiply)
          , left (NULL)
          , right (NULL)
    {
    }
};

class CppCond_MultiplicativeExpression_Divide : public CppCond_MultiplicativeExpression
{
public:
    CppCond_MultiplicativeExpression *left;
    CppCond_UnaryExpression *right;

    CppCond_MultiplicativeExpression_Divide ()
        : CppCond_MultiplicativeExpression (CppCond_MultiplicativeExpression::_Divide)
          , left (NULL)
          , right (NULL)
    {
    }
};

class CppCond_MultiplicativeExpression_Remainder : public CppCond_MultiplicativeExpression
{
public:
    CppCond_MultiplicativeExpression *left;
    CppCond_UnaryExpression *right;

    CppCond_MultiplicativeExpression_Remainder ()
        : CppCond_MultiplicativeExpression (CppCond_MultiplicativeExpression::_Remainder)
          , left (NULL)
          , right (NULL)
    {
    }
};

void dump_cpp_cond_MultiplicativeExpression (CppCond_MultiplicativeExpression const *, MyCpp::Size nest_level = 0);

class CppCond_AdditiveExpression : public CppCondElement
{
public:
    enum Type {
        _Expression,
        _Plus,
        _Minus
    };

    const Type additive_expression_type;

    CppCond_AdditiveExpression (Type type)
        : CppCondElement (CppCondElement::_AdditiveExpression),
          additive_expression_type (type)
    {
    }
};

class CppCond_AdditiveExpression_Expression : public CppCond_AdditiveExpression
{
public:
    CppCond_MultiplicativeExpression *expression;

    CppCond_AdditiveExpression_Expression ()
        : CppCond_AdditiveExpression (CppCond_AdditiveExpression::_Expression)
          , expression (NULL)
    {
    }
};

class CppCond_AdditiveExpression_Plus : public CppCond_AdditiveExpression
{
public:
    CppCond_AdditiveExpression *left;
    CppCond_MultiplicativeExpression *right;

    CppCond_AdditiveExpression_Plus ()
        : CppCond_AdditiveExpression (CppCond_AdditiveExpression::_Plus)
          , left (NULL)
          , right (NULL)
    {
    }
};

class CppCond_AdditiveExpression_Minus : public CppCond_AdditiveExpression
{
public:
    CppCond_AdditiveExpression *left;
    CppCond_MultiplicativeExpression *right;

    CppCond_AdditiveExpression_Minus ()
        : CppCond_AdditiveExpression (CppCond_AdditiveExpression::_Minus)
          , left (NULL)
          , right (NULL)
    {
    }
};

void dump_cpp_cond_AdditiveExpression (CppCond_AdditiveExpression const *, MyCpp::Size nest_level = 0);

class CppCond_ShiftExpression : public CppCondElement
{
public:
    enum Type {
        _Expression,
        _LeftShift,
        _RightShift
    };

    const Type shift_expression_type;

    CppCond_ShiftExpression (Type type)
        : CppCondElement (CppCondElement::_ShiftExpression),
          shift_expression_type (type)
    {
    }
};

class CppCond_ShiftExpression_Expression : public CppCond_ShiftExpression
{
public:
    CppCond_AdditiveExpression *expression;

    CppCond_ShiftExpression_Expression ()
        : CppCond_ShiftExpression (CppCond_ShiftExpression::_Expression)
          , expression (NULL)
    {
    }
};

class CppCond_ShiftExpression_LeftShift : public CppCond_ShiftExpression
{
public:
    CppCond_ShiftExpression *left;
    CppCond_AdditiveExpression *right;

    CppCond_ShiftExpression_LeftShift ()
        : CppCond_ShiftExpression (CppCond_ShiftExpression::_LeftShift)
          , left (NULL)
          , right (NULL)
    {
    }
};

class CppCond_ShiftExpression_RightShift : public CppCond_ShiftExpression
{
public:
    CppCond_ShiftExpression *left;
    CppCond_AdditiveExpression *right;

    CppCond_ShiftExpression_RightShift ()
        : CppCond_ShiftExpression (CppCond_ShiftExpression::_RightShift)
          , left (NULL)
          , right (NULL)
    {
    }
};

void dump_cpp_cond_ShiftExpression (CppCond_ShiftExpression const *, MyCpp::Size nest_level = 0);

class CppCond_RelationalExpression : public CppCondElement
{
public:
    enum Type {
        _Expression,
        _Less,
        _Greater,
        _LessOrEqual,
        _GreaterOrEqual
    };

    const Type relational_expression_type;

    CppCond_RelationalExpression (Type type)
        : CppCondElement (CppCondElement::_RelationalExpression),
          relational_expression_type (type)
    {
    }
};

class CppCond_RelationalExpression_Expression : public CppCond_RelationalExpression
{
public:
    CppCond_ShiftExpression *expression;

    CppCond_RelationalExpression_Expression ()
        : CppCond_RelationalExpression (CppCond_RelationalExpression::_Expression)
          , expression (NULL)
    {
    }
};

class CppCond_RelationalExpression_Less : public CppCond_RelationalExpression
{
public:
    CppCond_RelationalExpression *left;
    CppCond_ShiftExpression *right;

    CppCond_RelationalExpression_Less ()
        : CppCond_RelationalExpression (CppCond_RelationalExpression::_Less)
          , left (NULL)
          , right (NULL)
    {
    }
};

class CppCond_RelationalExpression_Greater : public CppCond_RelationalExpression
{
public:
    CppCond_RelationalExpression *left;
    CppCond_ShiftExpression *right;

    CppCond_RelationalExpression_Greater ()
        : CppCond_RelationalExpression (CppCond_RelationalExpression::_Greater)
          , left (NULL)
          , right (NULL)
    {
    }
};

class CppCond_RelationalExpression_LessOrEqual : public CppCond_RelationalExpression
{
public:
    CppCond_RelationalExpression *left;
    CppCond_ShiftExpression *right;

    CppCond_RelationalExpression_LessOrEqual ()
        : CppCond_RelationalExpression (CppCond_RelationalExpression::_LessOrEqual)
          , left (NULL)
          , right (NULL)
    {
    }
};

class CppCond_RelationalExpression_GreaterOrEqual : public CppCond_RelationalExpression
{
public:
    CppCond_RelationalExpression *left;
    CppCond_ShiftExpression *right;

    CppCond_RelationalExpression_GreaterOrEqual ()
        : CppCond_RelationalExpression (CppCond_RelationalExpression::_GreaterOrEqual)
          , left (NULL)
          , right (NULL)
    {
    }
};

void dump_cpp_cond_RelationalExpression (CppCond_RelationalExpression const *, MyCpp::Size nest_level = 0);

class CppCond_EqualityExpression : public CppCondElement
{
public:
    enum Type {
        _Expression,
        _Equal,
        _NotEqual
    };

    const Type equality_expression_type;

    CppCond_EqualityExpression (Type type)
        : CppCondElement (CppCondElement::_EqualityExpression),
          equality_expression_type (type)
    {
    }
};

class CppCond_EqualityExpression_Expression : public CppCond_EqualityExpression
{
public:
    CppCond_RelationalExpression *expression;

    CppCond_EqualityExpression_Expression ()
        : CppCond_EqualityExpression (CppCond_EqualityExpression::_Expression)
          , expression (NULL)
    {
    }
};

class CppCond_EqualityExpression_Equal : public CppCond_EqualityExpression
{
public:
    CppCond_EqualityExpression *left;
    CppCond_RelationalExpression *right;

    CppCond_EqualityExpression_Equal ()
        : CppCond_EqualityExpression (CppCond_EqualityExpression::_Equal)
          , left (NULL)
          , right (NULL)
    {
    }
};

class CppCond_EqualityExpression_NotEqual : public CppCond_EqualityExpression
{
public:
    CppCond_EqualityExpression *left;
    CppCond_RelationalExpression *right;

    CppCond_EqualityExpression_NotEqual ()
        : CppCond_EqualityExpression (CppCond_EqualityExpression::_NotEqual)
          , left (NULL)
          , right (NULL)
    {
    }
};

void dump_cpp_cond_EqualityExpression (CppCond_EqualityExpression const *, MyCpp::Size nest_level = 0);

class CppCond_AndExpression : public CppCondElement
{
public:
    enum Type {
        _Expression,
        _And
    };

    const Type and_expression_type;

    CppCond_AndExpression (Type type)
        : CppCondElement (CppCondElement::_AndExpression),
          and_expression_type (type)
    {
    }
};

class CppCond_AndExpression_Expression : public CppCond_AndExpression
{
public:
    CppCond_EqualityExpression *expression;

    CppCond_AndExpression_Expression ()
        : CppCond_AndExpression (CppCond_AndExpression::_Expression)
          , expression (NULL)
    {
    }
};

class CppCond_AndExpression_And : public CppCond_AndExpression
{
public:
    CppCond_AndExpression *left;
    CppCond_EqualityExpression *right;

    CppCond_AndExpression_And ()
        : CppCond_AndExpression (CppCond_AndExpression::_And)
          , left (NULL)
          , right (NULL)
    {
    }
};

void dump_cpp_cond_AndExpression (CppCond_AndExpression const *, MyCpp::Size nest_level = 0);

class CppCond_ExclusiveOrExpression : public CppCondElement
{
public:
    enum Type {
        _Expression,
        _ExclusiveOr
    };

    const Type exclusive_or_expression_type;

    CppCond_ExclusiveOrExpression (Type type)
        : CppCondElement (CppCondElement::_ExclusiveOrExpression),
          exclusive_or_expression_type (type)
    {
    }
};

class CppCond_ExclusiveOrExpression_Expression : public CppCond_ExclusiveOrExpression
{
public:
    CppCond_AndExpression *expression;

    CppCond_ExclusiveOrExpression_Expression ()
        : CppCond_ExclusiveOrExpression (CppCond_ExclusiveOrExpression::_Expression)
          , expression (NULL)
    {
    }
};

class CppCond_ExclusiveOrExpression_ExclusiveOr : public CppCond_ExclusiveOrExpression
{
public:
    CppCond_ExclusiveOrExpression *left;
    CppCond_AndExpression *right;

    CppCond_ExclusiveOrExpression_ExclusiveOr ()
        : CppCond_ExclusiveOrExpression (CppCond_ExclusiveOrExpression::_ExclusiveOr)
          , left (NULL)
          , right (NULL)
    {
    }
};

void dump_cpp_cond_ExclusiveOrExpression (CppCond_ExclusiveOrExpression const *, MyCpp::Size nest_level = 0);

class CppCond_InclusiveOrExpression : public CppCondElement
{
public:
    enum Type {
        _Expression,
        _InclusiveOr
    };

    const Type inclusive_or_expression_type;

    CppCond_InclusiveOrExpression (Type type)
        : CppCondElement (CppCondElement::_InclusiveOrExpression),
          inclusive_or_expression_type (type)
    {
    }
};

class CppCond_InclusiveOrExpression_Expression : public CppCond_InclusiveOrExpression
{
public:
    CppCond_ExclusiveOrExpression *expression;

    CppCond_InclusiveOrExpression_Expression ()
        : CppCond_InclusiveOrExpression (CppCond_InclusiveOrExpression::_Expression)
          , expression (NULL)
    {
    }
};

class CppCond_InclusiveOrExpression_InclusiveOr : public CppCond_InclusiveOrExpression
{
public:
    CppCond_InclusiveOrExpression *left;
    CppCond_ExclusiveOrExpression *right;

    CppCond_InclusiveOrExpression_InclusiveOr ()
        : CppCond_InclusiveOrExpression (CppCond_InclusiveOrExpression::_InclusiveOr)
          , left (NULL)
          , right (NULL)
    {
    }
};

void dump_cpp_cond_InclusiveOrExpression (CppCond_InclusiveOrExpression const *, MyCpp::Size nest_level = 0);

class CppCond_LogicalAndExpression : public CppCondElement
{
public:
    enum Type {
        _Expression,
        _LogicalAnd
    };

    const Type logical_and_expression_type;

    CppCond_LogicalAndExpression (Type type)
        : CppCondElement (CppCondElement::_LogicalAndExpression),
          logical_and_expression_type (type)
    {
    }
};

class CppCond_LogicalAndExpression_Expression : public CppCond_LogicalAndExpression
{
public:
    CppCond_InclusiveOrExpression *expression;

    CppCond_LogicalAndExpression_Expression ()
        : CppCond_LogicalAndExpression (CppCond_LogicalAndExpression::_Expression)
          , expression (NULL)
    {
    }
};

class CppCond_LogicalAndExpression_LogicalAnd : public CppCond_LogicalAndExpression
{
public:
    CppCond_LogicalAndExpression *left;
    CppCond_InclusiveOrExpression *right;

    CppCond_LogicalAndExpression_LogicalAnd ()
        : CppCond_LogicalAndExpression (CppCond_LogicalAndExpression::_LogicalAnd)
          , left (NULL)
          , right (NULL)
    {
    }
};

void dump_cpp_cond_LogicalAndExpression (CppCond_LogicalAndExpression const *, MyCpp::Size nest_level = 0);

class CppCond_LogicalOrExpression : public CppCondElement
{
public:
    enum Type {
        _LogicalOr,
        _Expression
    };

    const Type logical_or_expression_type;

    CppCond_LogicalOrExpression (Type type)
        : CppCondElement (CppCondElement::_LogicalOrExpression),
          logical_or_expression_type (type)
    {
    }
};

class CppCond_LogicalOrExpression_LogicalOr : public CppCond_LogicalOrExpression
{
public:
    CppCond_LogicalOrExpression *left;
    CppCond_LogicalAndExpression *right;

    CppCond_LogicalOrExpression_LogicalOr ()
        : CppCond_LogicalOrExpression (CppCond_LogicalOrExpression::_LogicalOr)
          , left (NULL)
          , right (NULL)
    {
    }
};

class CppCond_LogicalOrExpression_Expression : public CppCond_LogicalOrExpression
{
public:
    CppCond_LogicalAndExpression *expression;

    CppCond_LogicalOrExpression_Expression ()
        : CppCond_LogicalOrExpression (CppCond_LogicalOrExpression::_Expression)
          , expression (NULL)
    {
    }
};

void dump_cpp_cond_LogicalOrExpression (CppCond_LogicalOrExpression const *, MyCpp::Size nest_level = 0);

class CppCond_ConditionalExpression : public CppCondElement
{
public:
    enum Type {
        _Conditional,
        _Expression
    };

    const Type conditional_expression_type;

    CppCond_ConditionalExpression (Type type)
        : CppCondElement (CppCondElement::_ConditionalExpression),
          conditional_expression_type (type)
    {
    }
};

class CppCond_ConditionalExpression_Conditional : public CppCond_ConditionalExpression
{
public:
    CppCond_LogicalOrExpression *conditionExpression;
    CppCond_ConditionalExpression *ifExpression;
    CppCond_ConditionalExpression *elseExpression;

    CppCond_ConditionalExpression_Conditional ()
        : CppCond_ConditionalExpression (CppCond_ConditionalExpression::_Conditional)
          , conditionExpression (NULL)
          , ifExpression (NULL)
          , elseExpression (NULL)
    {
    }
};

class CppCond_ConditionalExpression_Expression : public CppCond_ConditionalExpression
{
public:
    CppCond_LogicalOrExpression *expression;

    CppCond_ConditionalExpression_Expression ()
        : CppCond_ConditionalExpression (CppCond_ConditionalExpression::_Expression)
          , expression (NULL)
    {
    }
};

void dump_cpp_cond_ConditionalExpression (CppCond_ConditionalExpression const *, MyCpp::Size nest_level = 0);

class CppCond_Grammar : public CppCondElement
{
public:
    CppCond_ConditionalExpression *expression;

    CppCond_Grammar ()
        : CppCondElement (CppCondElement::_Grammar)
          , expression (NULL)
    {
    }
};

MyCpp::Ref<Pargen::Grammar> create_cpp_cond_grammar ();

void dump_cpp_cond_grammar (CppCond_Grammar const *el);

}

#endif /* __SCRUFFY__CPP_COND_PARGEN_H__ */

